#!/usr/local/bin/perl -w
use strict;
use Getopt::Long;

###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
## Date: 2006-2007
## Goal: Prepare .snps and .haplo file for MACH
my $location = $0;
my $title = " -- ";

print "$title\n";
print "\t\@: $location\n";
print "\tby: Yun Li(ylwtx\@umich.edu)\n\n";

## Input Parameters
##

## Input Files
#   HapMap website files
#	(1) .phased
#	(2) _legend (header included)

## Output Files
#	(1) .haplo (haplotypes in 1234(ACGT) instead of 0 and 1)
#	(2) corresponding .snps file from _legend

################# END OF DOCUMENTATION ########################
###############################################################
my $MIN_NUM_OPTS = 3;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
        &usage();
        die "Failed to parse parameters\n\n";
}



my %opts = ();
  $opts{startpos} = 0;
  $opts{endpos} = 500000000;
  $opts{coding} = "1234";
my $a0index = 2;
my $a1index = 3;
my $col = 1;
my $startMarkerIndex = 0;
my $endMarkerIndex = 0;

Getopt::Long::GetOptions(\%opts,qw(
	p=s
	l=s
	o=s
	startpos=i
	endpos=i
	coding=s
)) || die "Failed to parse options\n\n";

my %codes = ( A => 1, C => 2, G => 3, T =>4 );
my @legendArr=();


#my $phased=$opts{p}.".phased";
#my $legend=$opts{l}."_legend.txt";
my $oh=$opts{o}.".haplo";
my $os=$opts{o}.".snps";
&printPar();

&main();

sub main
{
	&loadLegend();
	&loadPhased();
}

sub loadLegend
{
	if (-e "$os")
	{
		print "File $os exists!\n\n";
		die;
	}
	
	open(OUT, ">>$os") || die "Can't write file $os\n\n";
	open(IN,$opts{l}) || die "Can't open file $opts{l}\n\n";
	my $firstLegendLine=<IN>; # header line in $opts{l};
	my $legendArrPos=0;
	my $linenum = 0;
	while(my $line=<IN>)
	{
		$linenum ++;
		chomp($line);
		$line =~ s/^\s+//;
		my @lineArr=(split(/\s+/,$line));
		if ($#lineArr < 3)
		{
			print "line $linenum in $opts{l} has fewer fields than expected 4 \n\n";
			die;
		}
		
		if ($opts{coding} eq "1234")
		{
			$legendArr[$legendArrPos]=$codes{$lineArr[$a0index]};
			$legendArrPos++;
			$legendArr[$legendArrPos]=$codes{$lineArr[$a1index]};
			$legendArrPos++;
		}
		elsif ($opts{coding} eq "ACGT")
		{
			$legendArr[$legendArrPos]=$lineArr[$a0index];
			$legendArrPos++;
			$legendArr[$legendArrPos]=$lineArr[$a1index];
			$legendArrPos++;
		}
		else
		{
			print "coding = $opts{coding} NOT allowed. choose \"1234\" or \"ACGT\"\n\n";
			die;
		}
		
		if ($lineArr[1] < $opts{startpos})
		{
			$startMarkerIndex ++;
			$endMarkerIndex ++;
			next;
		}
		
		if ($lineArr[1] <= $opts{endpos})
		{
			$endMarkerIndex ++;
			print OUT "$lineArr[0]\n";
		}
		
		if ($lineArr[1] > $opts{endpos})
		{
			last;
		}
	}
	close(IN);
	close(OUT);
	#&printLegendArr();
	$endMarkerIndex --; # b/c off by one;
	print "Start Marker Index = $startMarkerIndex \nEnd Marker Index = $endMarkerIndex\n\n";
}


sub loadPhased
{
	if (-e "$oh")
	{
		print "File $oh exists!\n\n";
		die;
	}
	
	open(IN,$opts{p}) || die "Can't open file $opts{p}\n\n";
	open(OUT, ">>$oh") || die "Can't write file $oh\n\n";
	my $linenum = 0;
	while(my $line=<IN>)
	{
		$linenum ++;
		chomp($line);
		$line =~ s/^\s+//;
		my @lineArr=(split(/\s+/,$line));
		if ($#lineArr < $endMarkerIndex)
		{
			print "line $linenum in $opts{p} has fewer fields than expected.\n\n";
			die;
		}
		
		my @newLineArr=();
		#for(my $i=($col-1); $i<=$#lineArr; $i++)
		for(my $i=$startMarkerIndex; $i<=$endMarkerIndex; $i++)
		{
			if($lineArr[$i]==0)
			{
				$newLineArr[$i]=$legendArr[2*$i];
			}
			elsif($lineArr[$i]==1)
			{
				$newLineArr[$i]=$legendArr[2*$i+1];
			}
		}
		for(my $i=$startMarkerIndex; $i<$endMarkerIndex; $i++)
		{
			print OUT "$newLineArr[$i]";
		}
		for(my $i=$endMarkerIndex; $i<=$endMarkerIndex; $i++)
		{
			print OUT "$newLineArr[$i]\n";
		}
	}
	close(OUT);
	close(IN);
}


# begin sub

sub usage
{
	print "Usage:\n\t-p\t\tphased file from HapMap Phasing(Required)\n";
	print "\t-l\t\tlegend from HapMap Phasing(Required)\n";
	print "\t-o\t\toutput prefix (Required)\n";
	print "\t-startpos\tStarting position (Optional; 0 [bp])\n";
	print "\t-endpos\t\tEnding position (Optional; 500,000,000 [bp])\n";
	print "\t-coding\t\tCoding Scheme (Optional; 1234 [default] or ACGT)\n";
	print "\n";
}

sub printPar
{
	print "Parameters:\n";
	print "\t.phased file from HapMap phasing:\n\t\t-p $opts{p}\n";
	print "\tlegend file from HapMap phasing:\n\t\t-l $opts{l}\n";
	print "\toutput phasing file:\n\t\t-o $opts{o}\n";
	print "\tStarting position (in bp):\n\t\t-startpos $opts{startpos}\n";
	print "\tEnding position (in bp):\n\t\t-endpos $opts{endpos}\n";
	print "\tCoding Scheme (in bp):\n\t\t-coding $opts{coding}\n";
	print "\n";
}

sub printLegendArr
{
	for(my  $i=0; $i<=$#legendArr; $i++)
	{
		print "$legendArr[$i]\n"
	}
}


# ./HapMapForMACH.pl -p ~/mach1/GW_IMP_HM_NancyC/CEU/HMP2/genotypes_chr22_CEU_r21_nr_fwd_phased -l ~/mach1/GW_IMP_HM_NancyC/CEU/HMP2/genotypes_chr22_CEU_r21_nr_fwd_legend.txt -o test22
# diff test22.snps ~/imputation_GWA/unionDat/r3_snps/chr22.r3.snps
  ## no difference
# cut -f3 -d ' ' ~/imputation_GWA/HMP2/r3_haplo/chr22.r3.haplo > bb
# diff test22.haplo bb
  ## no difference


  
# ./HapMapForMACH.pl -p ~/mach1/GW_IMP_HM_NancyC/CEU/HMP2/genotypes_chr14_CEU_r21_nr_fwd_phased -l ~/mach1/GW_IMP_HM_NancyC/CEU/HMP2/genotypes_chr14_CEU_r21_nr_fwd_legend.txt -startpos 51102810 -endpos 70931456 -o test14_20Mb
# diff test14_20Mb.snps ~/imputation_GWA/allHMP_greedy/round.3/chr14/mle-mach1/ped/20Mb.snps
  ## no difference
# diff test14_20Mb.haplo ~/imputation_GWA/allHMP_greedy/round.3/chr14/mle-mach1/ped/20Mb.haplo
  ## no difference


